# frozen_string_literal: true

module Dynamo
  module Record
    class BatchRequest
      MAX_RECORD_SIZE = 400_000 # 400 KB

      attr_reader :record, :type

      def initialize(record, type)
        @type = type
        @record = record
        validate_type
        validate_request_size
      end

      def request
        @request ||= send("#{type}_request")
      end

      def save_request
        {
          put_request: {
            item: record.send(:_build_item_for_save)
          }
        }
      end

      def delete_request
        {
          delete_request: {
            key: record.send(:key_values)
          }
        }
      end

      def request_size
        @request_size ||= request.to_json.length
      end

      def validate_request_size
        return unless request_size > MAX_RECORD_SIZE

        raise RecordTooLargeError.new(record), 'Record is too large'
      end

      def validate_type
        return if %i[save delete].include? type

        raise UnsupportedRequestTypeError
      end
    end

    class UnsupportedRequestTypeError < RuntimeError; end

    class RecordTooLargeError < RuntimeError
      attr_reader :record

      def initialize(record)
        super
        @record = record
      end
    end
  end
end
