# frozen_string_literal: true

module Dynamo
  module Record
    module TaskHelpers
      class Cleanup
        def self.run
          raise 'Task not available on production' if Rails.env.production?

          Dir[Rails.root.join('app/models/*.rb').to_s].each do |filename|
            delete_by_class(filename)
          end
        end

        def self.delete_by_class(filename)
          klass = File.basename(filename, '.rb').camelize.constantize
          return unless klass.included_modules.include? Dynamo::Record::Model

          Rails.logger.info "Deleting all items in table: #{klass}"
          klass.scan.each(&:delete!)
        end
      end
    end
  end
end
